#ifndef CETONRTP_H
#define CETONRTP_H

#include <QObject>
#include <QUdpSocket>

#include "cetonrtsp.h"

struct RtpFixedHeader
{
    uint csrccount      : 4;
    uint extension      : 1;
    uint padding        : 1;
    uint version        : 2;

    uint payloadtype    : 7;
    uint marker         : 1;

    uint sequencenumber : 16;
    uint timestamp      : 32;
    uint ssrc           : 32;
};

class CetonRTP : public QObject
{
    Q_OBJECT
public:
    explicit CetonRTP(const QString& ip, uint tuner, ushort listenPort = 0, ushort controlPort = 8554, QObject *parent = 0);

    bool initialize();

    bool beginStreaming();
    bool endStreaming();

    int read(char *buffer, int bufferSize);

private:
    void processIncomingData();
    bool findRtpPayload(char* packet, char* &payload, int &payloadSize, int &sequenceNumber);


private:
    QString         _ip;
    uint            _tuner;
    uint            _listenPort;

    QUdpSocket      _rtpSocket;
    QUdpSocket      _dummySocket;
    QHostAddress    _acceptFrom;
    CetonRTSP       _rtsp;

    QByteArray      _readBuffer;
    QByteArray      _mpegBuffer;

    int             _lastRead;
    int             _thisRead;
    int             _lastSequenceNumber;
};

#endif // CETONRTP_H
