// -*- Mode: c++ -*-

#ifndef _CETONSTREAMHANDLER_H_
#define _CETONSTREAMHANDLER_H_

#include <vector>
using namespace std;

#include <QString>
#include <QMap>
#include <QMutex>
#include <QHttp>
#include <QUrl>

#include "util.h"
#include "DeviceReadBuffer.h"
#include "mpegstreamdata.h"
#include "dtvconfparserhelpers.h"

class CetonRecorder;
class CetonStreamHandler;
class CetonHttpRequest;
class DTVSignalMonitor;
class CetonChannel;
class DeviceReadBuffer;


class CetonHttpRequest : protected QThread
{
public:
    CetonHttpRequest(const QString &host, const QString &method, const QString &script, const QUrl &params);

    bool request();
    QString getResult() { return _response; };
    QString getResult(int &status_code) { status_code = _status_code; return _response; };

protected:
    void run();

private:
    bool    _error;
    int     _status_code;
    QString _response;

    QString _host;
    QString _method;
    QString _script;
    QUrl    _params;
 };


class CetonStreamHandler : public ReaderPausedCB
{
    friend void *run_ceton_stream_handler_thunk(void *param);

public:
    static CetonStreamHandler *Get(const QString &devicename);
    static void Return(CetonStreamHandler * & ref);

    void AddListener(MPEGStreamData *data);
    void RemoveListener(MPEGStreamData *data);

    bool IsRunning(void) const { return _running; }
    bool IsConnected(void) const;
    bool IsCableCardInstalled() const { return _using_cablecard; };

    // Commands
    bool EnterPowerSavingMode(void);
    bool TuneQAM(uint frequency, const QString &modulation, uint program);
    bool TuneCableCard(uint vchannel, int &program);

    bool TuneFrequency(uint frequency, const QString &modulation);
    bool TuneProgram(uint program);
    bool TuneVChannel(uint vchannel);
    uint GetProgramNumber();

private:
    bool VerifyTuning();
    void RepeatTuning();

    bool TunerOff();
    QString GetVar(const QString &section, const QString &variable);
    QStringList GetProgramList();
    bool HttpRequest(const QString& method, const QString &script, const QUrl &params,
                     QString &response, uint &status_code);

    // ReaderPausedCB
    virtual void ReaderPaused(int fd) { (void) fd; }

  private:
    CetonStreamHandler(const QString &);
    ~CetonStreamHandler();

    bool Connect(void);

    bool Open(void);
    void Close(void);

    void Start(void);
    void Stop(void);

    void Run(void);
    void RunTS(void);

    void SetRunning(bool);

  private:
    QString     _devicename;
    QString     _ip_address;
    bool        _using_rtp;
    uint        _card;
    uint        _tuner;
    QString     _device_path;
    bool        _using_cablecard;
    bool        _connected;
    bool        _valid;

    uint        _last_frequency;
    QString     _last_modulation;
    uint        _last_program;
    uint        _last_vchannel;
    QTime       _read_timer;

    static QMap<QString, bool>  _info_queried;

    mutable QMutex    _start_stop_lock;
    bool              _running;
    QWaitCondition    _running_state_changed;
    pthread_t         _reader_thread;

    vector<uint>      _eit_pids;

    mutable QMutex          _listener_lock;
    vector<MPEGStreamData*> _stream_data_list;

    // for implementing Get & Return
    static QMutex                            _handlers_lock;
    static QMap<QString, CetonStreamHandler*> _handlers;
    static QMap<QString, uint>               _handlers_refcnt;

};

#endif // _CETONSTREAMHANDLER_H_
